## Modelo com efeito aleatório

library(tidyverse)

## Definição do modelo
## Dados simulados do modelo:
## Y_{ij} \sim N(\mu_{i}, \sigma^2_y)
##     mu_{i} = beta_0 + b_{i}
##     b_{i} \sim N(0, \sigma^2_b)
## que, por ser normal (com ligação identidade)
## pode ser escrito por:
## Y_{ij} = \beta_0 + b_{i} + \epsilon_{ij}

## Simulando dados para o ajuste do modelo
Ngr <-  25
Nobs <- 10
set.seed(12)
sim <- data.frame(
    gr  = rep(1:Ngr, each = Nobs), #25 grupos com 10 observações
    bs  = rep(rnorm(Ngr, 0, sd = 10), each = Nobs), #25 efeitos aleatórios
    eps = rnorm(Ngr * Nobs, 0, sd = 4) #250 epsilons
)
sim <- transform(sim, y = 100 + bs + eps)
head(sim, 25)

boxplot(y ~ gr, data = sim)
abline(h = mean(sim$y), col = 2, lty = 2, lwd = 2)

##----------------------------------------------------------------------
## Estimativas "naive"
res <- sim %>%
    group_by(gr) %>%
    summarise(med = mean(y),
              var = var(y),
              sd = sd(y))
print(res, n = 25)
## bs naive
var(res$med)
sd(res$med)
## eps naive
mean(res$var)
sqrt(mean(res$var))

mean(sim$y)

##----------------------------------------------------------------------
## Modelo de efeitos aleatórios, modelos que utilizam a verossimilhança
## do modelo
require(lme4)

fit.lmer <- lmer(y ~ 1|gr, data = sim)
summary(fit.lmer)
## Efeito fixo (média geral)
fixef(fit.lmer)
## Efeitos aleatórios (desvios em relação ao efeito fixo)
ranef(fit.lmer)
## Fixo + aleatório = coeficientes
(coef.lmer <- (fixef(fit.lmer) + ranef(fit.lmer)$gr)[, 1])
cbind(coef(fit.lmer)$gr, coef.lmer, res$med)

mean(ranef(fit.lmer)$gr[,1]) # média dos efeitos
var(ranef(fit.lmer)$gr[,1])  # variância dos efeitos

##----------------------------------------------------------------------
## Utilizando inferência bayesiana para ajuste do modelo de efeito
## aleatório - JAGS
require(rjags)

## Do objeto sim, utilizou-se apenas as colunas grupo (gr) e valores
## obsservados (y), e formou-se uma lista com gr e y, recebidos pelo
## objeto sim.lst
sim.lst <- as.list(sim[c("gr", "y")])
## Inserindo o número de observações
sim.lst$N <- nrow(sim)
## Inserindo o número de grupos
sim.lst$Ngr <- length(unique(sim$gr))
str(sim.lst)

cat("model{
    ## Para o indivíduo j, j = 1, ..., N,
    for(j in 1:N){
      y[j] ~ dnorm(mu[gr[j]], tau.e)
    }
    ## Para o grupo i, i = 1, ..., Ngr
    for(i in 1:Ngr){
      mu[i] ~ dnorm(beta0, tau.b)
    }
    beta0 ~ dnorm(0, 1.0E-6)
    tau.b ~ dgamma(0.001, 0.001)
    sigma2.b <- 1/tau.b
    tau.e ~ dgamma(0.001, 0.001)
    sigma2.e <- 1/tau.e
    }", file="sim.jags")

sim.jags <- jags.model(file = "sim.jags",
                       data = sim.lst, n.chains = 3, n.adapt = 1000)

fit.jags <- coda.samples(
    sim.jags,
    variable.names = c("beta0", "sigma2.b", "sigma2.e"),
    n.iter = 10000,
    thin=10
)

summary(fit.jags)
plot(fit.jags)

## Posteriores
post <- do.call(rbind.data.frame, fit.jags)
str(post)
plot(post)

post.stk <- stack(post)
str(post.stk)



##----------------------------------------------------------------------
## Usando BRMS
fit.brms <- brm(y ~ 1 | gr, data = sim, cores = 4)

summary(fit.brms)
plot(fit.brms)
pairs(fit.brms)

posterior_summary(fit.brms)
posterior_interval(fit.brms)

prior_summary(fit.brms)
fixef(fit.brms)
ranef(fit.brms)

post.brms <- as_draws_df(fit.brms)
str(post.brms)

mcmc_plot(fit.brms)
mcmc_plot(fit.brms, variable = c("sd_gr__Intercept", "sigma"))
mcmc_plot(fit.brms, variable = c("sd_gr__Intercept", "sigma"),
          type = "acf")
mcmc_plot(fit.brms, type = "hist")

stancode(fit.brms)
launch_shinystan(fit.brms)

##----------------------------------------------------------------------
## Utilizando inferência bayesiana para ajuste do modelo de efeito
## aleatório - INLA
library(INLA)

fit.inla <- inla(y ~ 1 + f(gr, model = "iid"), data = sim,
                 control.compute = list(config = TRUE))
summary(fit.inla)
plot(fit.inla)

fit.inla$summary.fixed
fit.inla$summary.hyperpar
fit.inla$summary.random

## Converte precisao para SD
str(fit.inla$internal.marginals.hyperpar)
marg.e <- fit.inla$internal.marginals.hyperpar[[1]]
marg.e.sd <- inla.tmarginal(function(x) 1/sqrt(exp(x)), marg.e)
str(marg.e.sd)
plot(marg.e.sd, type = "l")
inla.zmarginal(marg.e.sd)

str(fit.inla$internal.marginals.hyperpar)
marg.b <- fit.inla$internal.marginals.hyperpar[[2]]
marg.b.sd <- inla.tmarginal(function(x) 1/sqrt(exp(x)), marg.b)
str(marg.b.sd)
plot(marg.b.sd, type = "l")
inla.zmarginal(marg.b.sd)

par(mfrow = c(1, 2))
plot(marg.e.sd, type = "l")
plot(marg.b.sd, type = "l")
par(mfrow = c(1, 1))

## Amostra da conjunta
m.inla.post <- inla.posterior.sample(n = 1e4, fit.inla)
str(m.inla.post, 2)
str(m.inla.post[[1]])
str(m.inla.post[[1]]$latent)
length(m.inla.post)
m.inla.post[[1]]$latent
m.inla.post[[1000]]$latent
str(m.inla.post[[1000]]$latent)

beta.post <- t(sapply(m.inla.post, function(x) x$latent[276]))
plot(density(beta.post))
